package javax.management.openmbean;

import java.util.Iterator;
import java.util.Collections;
import java.util.Set;
import java.util.TreeMap;

public class CompositeType extends OpenType<CompositeData>
{
    static final long serialVersionUID = -5366242454346948798L;
    private TreeMap<String, String> nameToDescription;
    private TreeMap<String, OpenType<?>> nameToType;
    private transient Integer myHashCode;
    private transient String myToString;
    private transient Set<String> myNamesSet;
    
    public CompositeType(String s, String s2, String[] array, String[] array2, OpenType<?>[] array3) throws OpenDataException {
        super(CompositeData.class.getName(), s, s2, false);
        this.myHashCode = null;
        this.myToString = null;
        this.myNamesSet = null;
        checkForNullElement(array, "itemNames");
        checkForNullElement(array2, "itemDescriptions");
        checkForNullElement(array3, "itemTypes");
        checkForEmptyString(array, "itemNames");
        checkForEmptyString(array2, "itemDescriptions");
        if (array.length != array2.length || array.length != array3.length) {
            throw new IllegalArgumentException("Array arguments itemNames[], itemDescriptions[] and itemTypes[] should be of same length (got " + array.length + ", " + array2.length + " and " + array3.length + ").");
        }
        this.nameToDescription = new TreeMap();
        this.nameToType = new TreeMap();
        int i = 0;
        while (i < array.length) {
            String trim = array[i].trim();
            if (this.nameToDescription.containsKey(trim)) {
                throw new OpenDataException("Argument's element itemNames[" + i + "]=\"" + array[i] + "\" duplicates a previous item names.");
            }
            this.nameToDescription.put(trim, array2[i].trim());
            this.nameToType.put(trim, array3[i]);
            ++i;
        }
        return;
    }
    
    private static void checkForNullElement(Object[] array, String s) {
        if (array == null || array.length == 0) {
            throw new IllegalArgumentException("Argument " + s + "[] cannot be null or empty.");
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] == null) {
                throw new IllegalArgumentException("Argument's element " + s + "[" + i + "] cannot be null.");
            }
            ++i;
        }
        return;
    }
    
    private static void checkForEmptyString(String[] array, String str) {
        int i = 0;
        while (i < array.length) {
            if (array[i].trim().equals("")) {
                throw new IllegalArgumentException("Argument's element " + str + "[" + i + "] cannot be an empty string.");
            }
            ++i;
        }
    }
    
    public boolean containsKey(String key) {
        return key != null && this.nameToDescription.containsKey(key);
    }
    
    public String getDescription(String key) {
        if (key == null) {
            return null;
        }
        return (String)this.nameToDescription.get(key);
    }
    
    public OpenType<?> getType(String key) {
        if (key == null) {
            return null;
        }
        return (OpenType)this.nameToType.get(key);
    }
    
    public Set<String> keySet() {
        if (this.myNamesSet == null) {
            this.myNamesSet = Collections.unmodifiableSet(this.nameToDescription.keySet());
        }
        return this.myNamesSet;
    }
    
    @Override
    public boolean isValue(Object o) {
        return o instanceof CompositeData && this.isAssignableFrom(((CompositeData)o).getCompositeType());
    }
    
    @Override
    boolean isAssignableFrom(OpenType<?> openType) {
        if (!(openType instanceof CompositeType)) {
            return false;
        }
        CompositeType compositeType = (CompositeType)openType;
        if (!compositeType.getTypeName().equals(this.getTypeName())) {
            return false;
        }
        for (String s : this.keySet()) {
            OpenType<Object> type = compositeType.getType(s);
            OpenType<Object> type2 = this.getType(s);
            if (type == null || !type2.isAssignableFrom(type)) {
                return false;
            }
            continue;
        }
        return true;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }
        CompositeType compositeType;
        try {
            compositeType = (CompositeType)o;
        }
        catch (ClassCastException ex) {
            return false;
        }
        return this.getTypeName().equals(compositeType.getTypeName()) && this.nameToType.equals(compositeType.nameToType);
    }
    
    @Override
    public int hashCode() {
        if (this.myHashCode == null) {
            int i = 0 + this.getTypeName().hashCode();
            for (String key : this.nameToDescription.keySet()) {
                i = i + key.hashCode() + ((OpenType)this.nameToType.get(key)).hashCode();
            }
            this.myHashCode = Integer.valueOf(i);
        }
        return (int)this.myHashCode;
    }
    
    @Override
    public String toString() {
        if (this.myToString == null) {
            StringBuilder sb = new StringBuilder();
            sb.append(this.getClass().getName());
            sb.append("(name=");
            sb.append(this.getTypeName());
            sb.append(",items=(");
            int n = 0;
            for (String s : this.nameToType.keySet()) {
                if (n > 0) {
                    sb.append(",");
                }
                sb.append("(itemName=");
                sb.append(s);
                sb.append(",itemType=");
                sb.append(((OpenType)this.nameToType.get(s)).toString() + ")");
                ++n;
            }
            sb.append("))");
            this.myToString = sb.toString();
        }
        return this.myToString;
    }
}
